/****************************************************************************
** Meta object code from reading C++ file 'majorimageprocessingthread.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/majorimageprocessingthread.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'majorimageprocessingthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26MajorImageProcessingThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto MajorImageProcessingThread::qt_create_metaobjectdata<qt_meta_tag_ZN26MajorImageProcessingThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MajorImageProcessingThread",
        "SendMajorImageProcessing",
        "",
        "QImage*",
        "image",
        "result",
        "SendFilterImageProcessing",
        "sigReflushSnapshotLabel",
        "sigYUVFrame",
        "uchar*",
        "yuv",
        "width",
        "height",
        "sigRenderYuv",
        "sigRecordFrame",
        "rgb",
        "size",
        "reachMaxDelayedFrames",
        "processingImage",
        "QImage&"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SendMajorImageProcessing'
        QtMocHelpers::SignalData<void(QImage *, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'SendFilterImageProcessing'
        QtMocHelpers::SignalData<void(QImage *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'sigReflushSnapshotLabel'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigYUVFrame'
        QtMocHelpers::SignalData<void(uchar *, uint, uint)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::UInt, 11 }, { QMetaType::UInt, 12 },
        }}),
        // Signal 'sigRenderYuv'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'sigRecordFrame'
        QtMocHelpers::SignalData<void(uchar *, uint)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { QMetaType::UInt, 16 },
        }}),
        // Signal 'reachMaxDelayedFrames'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'processingImage'
        QtMocHelpers::SlotData<void(QImage &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MajorImageProcessingThread, qt_meta_tag_ZN26MajorImageProcessingThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MajorImageProcessingThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26MajorImageProcessingThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26MajorImageProcessingThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26MajorImageProcessingThreadE_t>.metaTypes,
    nullptr
} };

void MajorImageProcessingThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MajorImageProcessingThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SendMajorImageProcessing((*reinterpret_cast<std::add_pointer_t<QImage*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->SendFilterImageProcessing((*reinterpret_cast<std::add_pointer_t<QImage*>>(_a[1]))); break;
        case 2: _t->sigReflushSnapshotLabel(); break;
        case 3: _t->sigYUVFrame((*reinterpret_cast<std::add_pointer_t<uchar*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3]))); break;
        case 4: _t->sigRenderYuv((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->sigRecordFrame((*reinterpret_cast<std::add_pointer_t<uchar*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 6: _t->reachMaxDelayedFrames(); break;
        case 7: _t->processingImage((*reinterpret_cast<std::add_pointer_t<QImage&>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)(QImage * , int )>(_a, &MajorImageProcessingThread::SendMajorImageProcessing, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)(QImage * )>(_a, &MajorImageProcessingThread::SendFilterImageProcessing, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)()>(_a, &MajorImageProcessingThread::sigReflushSnapshotLabel, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)(uchar * , uint , uint )>(_a, &MajorImageProcessingThread::sigYUVFrame, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)(bool )>(_a, &MajorImageProcessingThread::sigRenderYuv, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)(uchar * , uint )>(_a, &MajorImageProcessingThread::sigRecordFrame, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MajorImageProcessingThread::*)()>(_a, &MajorImageProcessingThread::reachMaxDelayedFrames, 6))
            return;
    }
}

const QMetaObject *MajorImageProcessingThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MajorImageProcessingThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26MajorImageProcessingThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int MajorImageProcessingThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void MajorImageProcessingThread::SendMajorImageProcessing(QImage * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void MajorImageProcessingThread::SendFilterImageProcessing(QImage * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MajorImageProcessingThread::sigReflushSnapshotLabel()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MajorImageProcessingThread::sigYUVFrame(uchar * _t1, uint _t2, uint _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void MajorImageProcessingThread::sigRenderYuv(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MajorImageProcessingThread::sigRecordFrame(uchar * _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void MajorImageProcessingThread::reachMaxDelayedFrames()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
