/****************************************************************************
** Meta object code from reading C++ file 'takephotosettingareawidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/takephotosettingareawidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'takephotosettingareawidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto takePhotoSettingAreaWidget::qt_create_metaobjectdata<qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "takePhotoSettingAreaWidget",
        "sngSetDelayTakePhoto",
        "",
        "delayTime",
        "sngSetFlashlight",
        "bFlashlightOn",
        "sngFilterChanged",
        "efilterType",
        "sngShowFilterName",
        "bShow",
        "sngSetFilterName",
        "name",
        "sigExposureChanged",
        "exposure",
        "sigFilterGroupDisplay",
        "bDisplay",
        "foldBtnClicked",
        "isShortcut",
        "unfoldBtnClicked",
        "flashlightFoldBtnClicked",
        "flashlightUnfoldBtnClicked",
        "delayUnfoldBtnClicked",
        "delayfoldBtnClicked",
        "filtersUnfoldBtnClicked",
        "filtersFoldBtnClicked",
        "onDelayBtnsClicked",
        "onFlashlightBtnsClicked",
        "onFilterBtnsClicked",
        "onUpdateFilterImage",
        "QImage*",
        "img",
        "onExposureValueChanged",
        "value",
        "exposureBtnClicked",
        "opacity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sngSetDelayTakePhoto'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'sngSetFlashlight'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'sngFilterChanged'
        QtMocHelpers::SignalData<void(efilterType)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 7 },
        }}),
        // Signal 'sngShowFilterName'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sngSetFilterName'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'sigExposureChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Signal 'sigFilterGroupDisplay'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'foldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'foldBtnClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'unfoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'unfoldBtnClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'flashlightFoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'flashlightFoldBtnClicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'flashlightUnfoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'flashlightUnfoldBtnClicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'delayUnfoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'delayUnfoldBtnClicked'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'delayfoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'delayfoldBtnClicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'filtersUnfoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'filtersUnfoldBtnClicked'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'filtersFoldBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'filtersFoldBtnClicked'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onDelayBtnsClicked'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'onDelayBtnsClicked'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onFlashlightBtnsClicked'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'onFlashlightBtnsClicked'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onFilterBtnsClicked'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'onFilterBtnsClicked'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onUpdateFilterImage'
        QtMocHelpers::SlotData<void(QImage *)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'onExposureValueChanged'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
        // Slot 'exposureBtnClicked'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'exposureBtnClicked'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'opacity'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<takePhotoSettingAreaWidget, qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject takePhotoSettingAreaWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t>.metaTypes,
    nullptr
} };

void takePhotoSettingAreaWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<takePhotoSettingAreaWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sngSetDelayTakePhoto((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->sngSetFlashlight((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->sngFilterChanged((*reinterpret_cast<std::add_pointer_t<efilterType>>(_a[1]))); break;
        case 3: _t->sngShowFilterName((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->sngSetFilterName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->sigExposureChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->sigFilterGroupDisplay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->foldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->foldBtnClicked(); break;
        case 9: _t->unfoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->unfoldBtnClicked(); break;
        case 11: _t->flashlightFoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->flashlightFoldBtnClicked(); break;
        case 13: _t->flashlightUnfoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->flashlightUnfoldBtnClicked(); break;
        case 15: _t->delayUnfoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->delayUnfoldBtnClicked(); break;
        case 17: _t->delayfoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->delayfoldBtnClicked(); break;
        case 19: _t->filtersUnfoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->filtersUnfoldBtnClicked(); break;
        case 21: _t->filtersFoldBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->filtersFoldBtnClicked(); break;
        case 23: _t->onDelayBtnsClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->onDelayBtnsClicked(); break;
        case 25: _t->onFlashlightBtnsClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->onFlashlightBtnsClicked(); break;
        case 27: _t->onFilterBtnsClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->onFilterBtnsClicked(); break;
        case 29: _t->onUpdateFilterImage((*reinterpret_cast<std::add_pointer_t<QImage*>>(_a[1]))); break;
        case 30: _t->onExposureValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->exposureBtnClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->exposureBtnClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(int )>(_a, &takePhotoSettingAreaWidget::sngSetDelayTakePhoto, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(bool )>(_a, &takePhotoSettingAreaWidget::sngSetFlashlight, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(efilterType )>(_a, &takePhotoSettingAreaWidget::sngFilterChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(bool )>(_a, &takePhotoSettingAreaWidget::sngShowFilterName, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(const QString & )>(_a, &takePhotoSettingAreaWidget::sngSetFilterName, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(int )>(_a, &takePhotoSettingAreaWidget::sigExposureChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (takePhotoSettingAreaWidget::*)(bool )>(_a, &takePhotoSettingAreaWidget::sigFilterGroupDisplay, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->getOpacity(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOpacity(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *takePhotoSettingAreaWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *takePhotoSettingAreaWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26takePhotoSettingAreaWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int takePhotoSettingAreaWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 33;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void takePhotoSettingAreaWidget::sngSetDelayTakePhoto(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void takePhotoSettingAreaWidget::sngSetFlashlight(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void takePhotoSettingAreaWidget::sngFilterChanged(efilterType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void takePhotoSettingAreaWidget::sngShowFilterName(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void takePhotoSettingAreaWidget::sngSetFilterName(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void takePhotoSettingAreaWidget::sigExposureChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void takePhotoSettingAreaWidget::sigFilterGroupDisplay(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
